# Sid Meier's Civilization 4
# Copyright Firaxis Games 2005
# Orion's Combined Forces Mod
from CvPythonExtensions import *
import CvUtil
import CvTechChooser
import traceback
import os
import sys
import PyHelpers
import pickle
import CvGameUtils
import CvEventInterface
import CvWBPopups
import Popup as PyPopup
import OVPromotionUtils
import math


# for C++ compatibility
false=False
true=True

# globals
gc = CyGlobalContext()
localText = CyTranslator()
PyPlayer = PyHelpers.PyPlayer
PyInfo = PyHelpers.PyInfo
PyCity = PyHelpers.PyCity
PyGame = PyHelpers.PyGame
ArtFileMgr = CyArtFileMgr()

dTITypeList = [
		"UNIT_HTI",
		"UNIT_MTI",
		"UNIT_NTI",
		"UNIT_TATI",
		"UNIT_SATI"
	]

dArmyList = [
		"UNIT_ANCIENT_ARMY",
		"UNIT_CLASSICAL_ARMY",
		"UNIT_MEDIEVAL_ARMY",
		"UNIT_RENAISSANCE_ARMY",
		"UNIT_INDUSTRIAL_ARMY",
		"UNIT_MODERN_ARMY",
		"UNIT_FUTURE_ARMY"
	]

dFleetList = [
		"UNIT_ANCIENT_FLEET",
		"UNIT_CLASSICAL_FLEET",
		"UNIT_MEDIEVAL_FLEET",
		"UNIT_RENAISSANCE_FLEET",
		"UNIT_INDUSTRIAL_FLEET",
		"UNIT_MODERN_FLEET",
		"UNIT_FUTURE_FLEET"
	]
	
dMaxUnitsData = {
		"ERA_ANCIENT": (3,0,0,0,0,0,0),
		"ERA_CLASSICAL": (4,3,0,0,0,0,0),
		"ERA_MEDIEVAL": (5,4,3,0,0,0,0),
		"ERA_RENAISSANCE": (6,5,4,3,0,0,0),
		"ERA_INDUSTRIAL": (7,6,5,4,3,0,0),
		"ERA_MODERN": (8,7,6,5,4,3,0),
		"ERA_FUTURE": (9,8,7,6,5,4,3)
	}	
	
dAirAssaultBrigadeList = [
		"UNITCOMBAT_HELICOPTER"
	]
	
dArmyCombatList = [
		"UNITCOMBAT_MELEE",
		"UNITCOMBAT_ARCHER",
		"UNITCOMBAT_MOUNTED",
		"UNITCOMBAT_GUN",
		"UNITCOMBAT_ARMOR"
	]
	
dNavyCombatList = [
		"UNITCOMBAT_NAVAL"
	]
	
dAirForceCombatList = [
		"UNITCOMBAT_AIR"
	]

dAirForcePromList = [
		"PROMOTION_AIR_FORCE_COMBAT1",
		"PROMOTION_AIR_FORCE_COMBAT2",
		"PROMOTION_AIR_FORCE_COMBAT3"
	]
	
dProhibitedAITypeList = [
		"UNITAI_WORKER",
		"UNITAI_WORKER_SEA",
		"UNITAI_SETTLE",
		"UNITAI_ASSAULT_SEA",
		"UNITAI_MERCHANT"
	]
		
# #######################################
#	Begin Combined Forces Functions		#
# #######################################

def getRequiredCitiesPerCombinedUnit():
	# Orion's Combined Forces Mod
	return 3

####################################################################	
def doTIPlacementAI(pUnit):
	# Orion's Combined Forces Mod
	# Sends Training Instructor unit to a city where the AI can build a Combined Unit
	iqUnitType = pUnit.getUnitType()	
	iOwner = pUnit.getOwner()
	AIpPlayer = gc.getPlayer(iOwner)
	iRefCityList = PyPlayer(iOwner).getCityList()
	AICan = False
	
	# All Training Instructors			
	if isTypeInList(iqUnitType, dTITypeList):	
		if not AIpPlayer.isNone() and not AIpPlayer.isBarbarian() and not AIpPlayer.isHuman( ) and AIpPlayer.isAlive():
			# Loop through Cities
			for pyCity in iRefCityList:
				pCity = pyCity.GetCy()
				
				if pCity != None:
					# Does AI city have enough units to build a Combined Unit?
					if canAIBuildCombinedUnit(iOwner, pCity, iqUnitType):
						AICity = pCity
						AICan = True
						break
	
	if AICan:
		if hasTIUnit(AICity, iOwner, iqUnitType):
			# Get the Selected Era for the units in the AI city
			AISelectedEra = getAICombinedUnitEra(iOwner, AICity, iqUnitType)
			doCombinedUnitPreChecks(iOwner, AISelectedEra, AICity, iqUnitType)
			
		# Find route to City
		if pUnit.generatePath(AICity.plot(), 0, False, None):
			# Get the Selected Era for the units in the AI city
			AISelectedEra = getAICombinedUnitEra(iOwner, AICity, iqUnitType)
						
			if pUnit.getX() != AICity.getX() or pUnit.getY() != AICity.getY():
				# Move the MTI to the AI city, where Combined Unit can be built.
				pUnit.getGroup().pushMission(MissionTypes.MISSION_MOVE_TO, AICity.getX(), AICity.getY(), 0, False, True, MissionAITypes.NO_MISSIONAI, pUnit.plot(), pUnit)
				
			else:
				doCombinedUnitPreChecks(iOwner, AISelectedEra, AICity, iqUnitType)
				
def doBuildCombinedUnit(pUnit):
	# Orion's Combined Forces Mod
	# Displays the Pick Era Popup for human player
	iUnitType = pUnit.getUnitType()
	iOwner = pUnit.getOwner()
	pPlayer = gc.getPlayer(iOwner)	
	iHTI = gc.getInfoTypeForString("UNIT_HTI")
	iMTI = gc.getInfoTypeForString("UNIT_MTI")
	iNTI = gc.getInfoTypeForString("UNIT_NTI")
	iTATI = gc.getInfoTypeForString("UNIT_TATI")
	iSATI = gc.getInfoTypeForString("UNIT_SATI")
	iCount = 0
	
	#if not pPlayer.isNone() and not pPlayer.isBarbarian() and pPlayer.isHuman( ) and pPlayer.isAlive():
	
	if not pPlayer.isNone() and not pPlayer.isBarbarian() and pPlayer.isAlive():
		# Let Human Player select the Era
		if pPlayer.isHuman():
			# Get the Current Era
			iCurrentEra = pPlayer.getCurrentEra()
		
			#Call up Pick Combined Unit Era Popup
			popupInfo = CyPopupInfo()
			popupInfo.setButtonPopupType(ButtonPopupTypes.BUTTONPOPUP_PYTHON)
			popupInfo.setData1(iOwner)
			popupInfo.setOnClickedPythonCallback("selectEraCallback")
			
			if iUnitType == iHTI:
				popupInfo.setData2(iHTI)
				szTitle = u"<font=4b>" + localText.getText("TXT_KEY_AIR_ASSAULT_BRIGADE_POPUP_TITLE", ()) + u"</font>"
				szBody = u"<font=3>" + localText.getText("TXT_KEY_SELECT_AIR_ASSAULT_BRIGADE_ERA", ()) + u"</font>"
				popupInfo.setText(szTitle + u"\n\n" + szBody)
			
			if iUnitType == iMTI:
				popupInfo.setData2(iMTI)
				szTitle = u"<font=4b>" + localText.getText("TXT_KEY_ARMY_POPUP_TITLE", ()) + u"</font>"
				szBody = u"<font=3>" + localText.getText("TXT_KEY_SELECT_ARMY_ERA", ()) + u"</font>"
				popupInfo.setText(szTitle + u"\n\n" + szBody)
				
			if iUnitType == iNTI:
				popupInfo.setData2(iNTI)
				szTitle = u"<font=4b>" + localText.getText("TXT_KEY_FLEET_POPUP_TITLE", ()) + u"</font>"
				szBody = u"<font=3>" + localText.getText("TXT_KEY_SELECT_FLEET_ERA", ()) + u"</font>"
				popupInfo.setText(szTitle + u"\n\n" + szBody)
			
			if iUnitType == iTATI:				
				popupInfo.setData2(iTATI)
				szTitle = u"<font=4b>" + localText.getText("TXT_KEY_TACTICAL_AIR_FORCE_POPUP_TITLE", ()) + u"</font>"
				szBody = u"<font=3>" + localText.getText("TXT_KEY_SELECT_TACTICAL_AIR_FORCE_ERA", ()) + u"</font>"	
			
			if iUnitType == iSATI:				
				popupInfo.setData2(iSATI)
				szTitle = u"<font=4b>" + localText.getText("TXT_KEY_STRATEGIC_AIR_FORCE_POPUP_TITLE", ()) + u"</font>"
				szBody = u"<font=3>" + localText.getText("TXT_KEY_SELECT_STRATEGIC_AIR_FORCE_ERA", ()) + u"</font>"	
			
			for i in range(gc.getNumEraInfos()):
				# Reads "Ancient"
				EraDescription = gc.getEraInfo(i).getDescription()
				# Reads "ERA_ANCIENT"
				EraType = gc.getEraInfo(i).getType()
				strButtonText = str(EraDescription)
				MyArt = "INTERFACE_" + str(EraType)
								
				if not i > iCurrentEra:
					iCount = iCount + 1
					popupInfo.addPythonButton(strButtonText, ArtFileMgr.getInterfaceArtInfo(MyArt).getPath())
					#CyInterface().addImmediateMessage(str(i), "")
					
			popupInfo.setData3(iCount)
			popupInfo.addPythonButton("Cancel", "")
			popupInfo.addPopup(iOwner)
			

		
def getTotalCombinedUnits(iPlayer):
	# Orion's Combined Forces Mod
	# Returns the total number of Combined units
	iCombinedUnitCount = 0
	pPlayer = gc.getPlayer(iPlayer)
	iRefUnitList = PyPlayer(iPlayer).getUnitList()
	iAirAssaultBrigadeCombatType = gc.getInfoTypeForString("UNITCOMBAT_UNITCOMBAT_AABRIGADE")
	iArmyCombatType = gc.getInfoTypeForString("UNITCOMBAT_ARMY")
	iNavyCombatType = gc.getInfoTypeForString("UNITCOMBAT_FLEET")
	iAirForce = gc.getInfoTypeForString("UNITCOMBAT_AIR")
	iAirCombatPromotion1 = gc.getInfoTypeForString("PROMOTION_AIR_FORCE_COMBAT1")
	iAirCombatPromotion2 = gc.getInfoTypeForString("PROMOTION_AIR_FORCE_COMBAT2")
	iAirCombatPromotion3 = gc.getInfoTypeForString("PROMOTION_AIR_FORCE_COMBAT3")
	
	for xUnit in iRefUnitList:
		xUnitType = xUnit.getUnitType()
		xUnitCombatType = xUnit.getUnitCombatType()		
		
		if xUnitCombatType == iAirAssaultBrigadeCombatType or xUnitCombatType == iArmyCombatType or xUnitCombatType == iNavyCombatType:
			iCombinedUnitCount += 1
			
		if xUnitType == iAirForce:
			if xUnit.isHasPromotion(iAirCombatPromotion1) or xUnit.isHasPromotion(iAirCombatPromotion2) or xUnit.isHasPromotion(iAirCombatPromotion3):
				iCombinedUnitCount += 1
				
		return iCombinedUnitCount		

				
def meetsTIConditions(pCity, iUnitType):
	# Orion's Combined Forces Mod
	# Returns True if the city meets the specified conditions for producing a Training Instructor
	# Returns False if the city does not meet the specified conditions for producing a Training Instructor
	DoesMeetTIConditions = False
	iOwner = pCity.getOwner()
	pPlayer = gc.getPlayer(iOwner)
	iOrganizedWarfareTech = gc.getInfoTypeForString("TECH_ORGANIZED_WARFARE")
	
	# Does the Player have the required Tech
	if gc.getTeam(pPlayer.getTeam()).isHasTech(iOrganizedWarfareTech):
		iBarracks = gc.getInfoTypeForString("BUILDING_BARRACKS")		
		# Does the city have a Barracks?
		if pCity.isHasBuilding(iBarracks):			
			# Must not have any iTIs
			if not hasUnitType(iOwner, iUnitType):
				pTotalCities = pPlayer.getNumCities()				
				# Must have at least 3 cities to build first Combined unit
				if pTotalCities > 2:
					TotalCombinedUnits = getTotalCombinedUnits(iOwner)
					# Must have default number of cities for each Combined Unit trained
					if TotalCombinedUnits * getRequiredCitiesPerCombinedUnit() <= pTotalCities:
						#CyInterface().addImmediateMessage("Can Train TI", "")
						DoesMeetTIConditions = True
						
	if DoesMeetTIConditions:
		return True
	
	return False

	
#def showBuildCombinedUnitButton(pUnit):
def showBuildCombinedUnitButton(pUnit):
	# Orion's Combined Forces Mod
	# Enables the Training Instructor Button to build a Combined Unit
	# Assumes the plot is a city	
	iOwner = pUnit.getOwner()	
	pUnitOwner = gc.getPlayer(iOwner)
	iUnitType = pUnit.getUnitType()	
	zShowButton = False
		
	
	if isTypeInList(iUnitType, dTITypeList):
		# Get the city
		pCity = gc.getMap().plot(pUnit.getX(), pUnit.getY()).getPlotCity()
		iCityOwner = pCity.getOwner()
		pCityPlayer = gc.getPlayer(iCityOwner)
		
		# Make sure the City owner matches the unit owner.
		if (iCityOwner == iOwner) or (gc.getTeam(pCityPlayer.getTeam()).isVassal(gc.getPlayer(iOwner).getTeam())):
			pTotalCities = pUnitOwner.getNumCities()			
			# Must have at least 3 cities to build first Combined unit
			if pTotalCities > 2:
				TotalCombinedUnits = getTotalCombinedUnits(iOwner)				
				# Must have default number of cities for each Combined Unit trained
				if TotalCombinedUnits * getRequiredCitiesPerCombinedUnit() <= pTotalCities:
					#CyInterface().addImmediateMessage("Show Button", "")
					zShowButton = True
	
	return zShowButton	
	

def getTIProductionChance(AIpPlayer):
	# Orion's Combined Forces Mod
	# Accelerates Training Instructor Production for Aggressive AI traits
	iTrait = gc.getInfoTypeForString("TRAIT_AGGRESSIVE")
	
	if (AIpPlayer.hasTrait(iTrait)):
		# Aggressive AI Players get a increased chance for Producing a TI
		return 50
	else:
		# Non Aggressive AI Players get a smaller chance for Producing a TI
		return 33


def isTypeInList(yType, dList):
	# Orion's Combined Forces Mod
	# Returns True if the yType is found in the dList
	# Returns False if the yType is not found in the dList
	bFoundType = False
	
	for szType in range(len(dList)):
		MyType = str(dList[szType])
		iType = gc.getInfoTypeForString(MyType)
				
		if yType == iType:
			#CyInterface().addImmediateMessage("Found It!", "")
			bFoundType = True
			break
	
	if bFoundType:
		return True
	else:
		return False


def getUnitEra(pUnitXXType):
	# Orion's Combined Forces Mod
	# Returns the Era for a specified Unit type
	# Identify a valid Unit
	if pUnitXXType != -1:
		# Add exception for Warriors, who have no prerequisite Tech
		if pUnitXXType == gc.getInfoTypeForString("UNIT_WARRIOR"):
			# This makes Warriors appear on the selection list to create Army units
			XiEra = 0
			return XiEra
		else:
			# Get the Unit Info
			pUnitInfo = gc.getUnitInfo(pUnitXXType)
			
			if pUnitInfo != -1:
				# Get the prerequisit Tech for the Unit
				iTechType = pUnitInfo.getPrereqAndTech()
				
				if iTechType != -1:
					# Get the Tech Era
					XiEra = gc.getTechInfo(iTechType).getEra()
					
					if XiEra != -1:
						return XiEra
			

def hasUnitType(iPlayer, iUnitType):
	# Orion's Combined Forces Mod
	# Returns True if the player has the specified unit type
	# Returns False if the player does not have the specified unit type
	xRefUnitList = PyPlayer(iPlayer).getUnitList()
	
	for pyUnit in xRefUnitList:		
		pyUnitType = pyUnit.getUnitType()
		
		if (pyUnitType == iUnitType):
			return True
			
	return False
	

def hasTIUnit(pCity, iActivePlayer, iTI):
	# Orion's Combined Forces Mod
	# Returns True if the plot has the specified Instructor unit (HTI, MTI, NTI or ATI)
	# Returns False if the plot does not have the specified Instructor unit (HTI, MTI, NTI or ATI)
	FoundTI = False
	iX = pCity.getX()
	iY = pCity.getY()
	pCityPlot = CyMap().plot(iX, iY)
		
	for iI in range (pCityPlot.getNumUnits()):
		# ID the Unit so we can check the type
		xUnit = pCityPlot.getUnit(iI)
		ixUnitOwner = xUnit.getOwner()
		xUnitType = xUnit.getUnitType()
						
		if xUnitType == iTI:
			if ixUnitOwner == iActivePlayer:
				#CyInterface().addImmediateMessage("Found iTI", "")
				FoundTI = True
				break
				
	if FoundTI:
		return True
	else:
		return False	

def getPlotAirUnitCount(pPlot, iUnitType, iMinUnitCount, iTI):		
	# Orion's Combined Forces Mod
	# Returns True if the plot has the min number of specified air unitTypes
	# Returns Flase if the plot does not have the min number of specified air unitTypes	
	iTotalTypeCount = 0
	iTATI = gc.getInfoTypeForString("UNIT_TATI")
	iSATI = gc.getInfoTypeForString("UNIT_SATI")
	
	for iUnitXLoop in range(pPlot.getNumUnits()):
		UnitX = pPlot.getUnit(iUnitXLoop)
		UnitXType = UnitX.getUnitType()
		UnitXIntercept = gc.getUnitInfo(UnitXType).getInterceptionProbability()
		
		if UnitXType == iUnitType:
			if iTI == iTATI:
				# Is this a Fighter
				if not UnitXIntercept == 0:
					iTotalTypeCount = iTotalTypeCount + 1
					if iTotalTypeCount == iMinUnitCount:
						return True
			
			elif iTI == iSATI:
				# Is this a Bomber
				if UnitXIntercept == 0:
					iTotalTypeCount = iTotalTypeCount + 1
					if iTotalTypeCount == iMinUnitCount:
						return True
				
	return False
	

#####################################################################################
def hasEnoughUnitsToCombine(iPlayer, iMaxUnits, iSelectedEra, gCity, iTI):
	# Orion's Combined Forces Mod
	# Returns True if the player has enough units to create a combined unit
	# Returns False if the player does not have enough units to create a combined unit
	iUnitCount = 0
	iHTI = gc.getInfoTypeForString("UNIT_HTI")
	iMTI = gc.getInfoTypeForString("UNIT_MTI")
	iNTI = gc.getInfoTypeForString("UNIT_NTI")
	iTATI = gc.getInfoTypeForString("UNIT_TATI")
	iSATI = gc.getInfoTypeForString("UNIT_SATI")
	# Get City Plot
	ipX = gCity.getX()
	ipY = gCity.getY()
	pCityPlot = CyMap().plot(ipX, ipY)
	AirUnits = False
	Fighters = False
	Bombers = False
	
	if iTI == iHTI:
		# Note The game does not consider Helicopters as AirUnits
		AirUnits = False
		MydCombatList = dAirAssaultBrigadeList
	
	if iTI == iMTI:
		AirUnits = False
		MydCombatList = dArmyCombatList
		
	if iTI == iNTI:
		AirUnits = False
		MydCombatList = dNavyCombatList
		
	if iTI == iTATI: 
		AirUnits = True
		Fighters = True
		#CyInterface().addImmediateMessage("A", "")
		MydCombatList = dAirForceCombatList
		
	if iTI == iSATI:
		AirUnits = True
		Bombers = True
		#CyInterface().addImmediateMessage("A", "")
		MydCombatList = dAirForceCombatList
	
	# Check each Unit on plot
	for iUnitLoop in range (pCityPlot.getNumUnits()):
		# Identify the Unit
		pUnitZ = pCityPlot.getUnit(iUnitLoop)
		pUnitZType = pUnitZ.getUnitType()		
		pUnitZCombatType = pUnitZ.getUnitCombatType()
		iOwner = pUnitZ.getOwner()
		
		# Make sure the player owns the unit.
		if iOwner == iPlayer:
			# Select only undamaged units
			if not pUnitZ.isHurt():
				# Exclude units not in Combat List
				if isTypeInList(pUnitZCombatType, MydCombatList):
					# Exclude units with prohibited AITypes
					if not isTypeInList(pUnitZType, dProhibitedAITypeList):
									
						# Get the Era for the Unit
						iEra = getUnitEra(pUnitZType)
						
						if AirUnits:					
							#CyInterface().addImmediateMessage("A", "")						
							if iEra == iSelectedEra:
								# If Tactical Air Training Instructor.
								if Fighters:
									# Get Intercept Probability
									pUnitZIntercept = gc.getUnitInfo(pUnitZType).getInterceptionProbability()
									# Look for Fighters
									if not pUnitZIntercept == 0:										
										# Does the plot have the min number of units for this pUnitZType
										if getPlotAirUnitCount(pCityPlot, pUnitZType, iMaxUnits, iTATI):
											#CyInterface().addImmediateMessage("Fighters", "")
											# Yup!  Can build a Tactical Air Force
											return True
							
								if Bombers:
									# Get Intercept Probability
									pUnitZIntercept = gc.getUnitInfo(pUnitZType).getInterceptionProbability()
									# Look for Bombers
									if pUnitZIntercept == 0:									
										# Does the plot have the min number of units for this pUnitZType
										if getPlotAirUnitCount(pCityPlot, pUnitZType, iMaxUnits, iSATI):
											#CyInterface().addImmediateMessage("Bombers", "")
											# Yup!  Can build a Strategic Air Force
											return True
					
						if not AirUnits:
							# Is the Unit Era equal to or less than the Selected Era
							if iEra <= iSelectedEra:
								# Yup!  Can build an Army, Navy or Brigade 
								iUnitCount = iUnitCount + 1
	
	if iUnitCount >= iMaxUnits:
		# Yup!  Can build an Army, Navy or Brigade
		#CyInterface().addImmediateMessage("Can Build Army, Navy or Brigade", "")
		return True
	
	return False

#####################################################################################

def canAIBuildCombinedUnit(iPlayer, AICity, iTI):
	# Orion's Combined Forces Mod
	# Returns True if the AI city has enough units to build the TI specified combined unit.
	# Returns False if the AI city does not have enough units to build the TI specified combined unit.
	pPlayer = gc.getPlayer(iPlayer)
	iCurrentEra = pPlayer.getCurrentEra()
	FoundEligibleAICity = False
	# Loop through Eras
	for i in range(gc.getNumEraInfos()):
		if not i > iCurrentEra:
			iMaxUnits = getMaxUnits(iCurrentEra, i)
			if hasEnoughUnitsToCombine(iPlayer, iMaxUnits, i, AICity, iTI):
				FoundEligibleAICity = True
				break
				
	if FoundEligibleAICity:
		#CyInterface().addImmediateMessage("Can Build 1", "")
		return True
	
	return False


def getAICombinedUnitEra(iPlayer, AICity, iTI):
	# Orion's Combined Forces Mod
	# Returns the selected Era for the AI 
	pPlayer = gc.getPlayer(iPlayer)
	iCurrentEra = pPlayer.getCurrentEra()
	FoundEra = False
	
	# Loop through Eras 
	for i in range(gc.getNumEraInfos()):
		if not i > iCurrentEra:
			iMaxUnits = getMaxUnits(iCurrentEra, i)
			
			if hasEnoughUnitsToCombine(iPlayer, iMaxUnits, i, AICity, iTI):
				#CyInterface().addImmediateMessage(str(i), "")
				return i
				break				

def doCombinedUnitCheckFailurePopup(iPlayer, iTI):
	# Orion's Combined Forces Mod
	# Displays the failure message
	# Failure Message occurs when the player does not have enough units to create the Combined Unit
	iHTI = gc.getInfoTypeForString("UNIT_HTI")
	iMTI = gc.getInfoTypeForString("UNIT_MTI")
	iNTI = gc.getInfoTypeForString("UNIT_NTI")
	iTATI = gc.getInfoTypeForString("UNIT_TATI")
	iSATI = gc.getInfoTypeForString("UNIT_SATI")
	
	popupInfo = CyPopupInfo()
	popupInfo.setButtonPopupType(ButtonPopupTypes.BUTTONPOPUP_PYTHON)
	
	if iTI == iHTI:
		szTitle = u"<font=4b>" + localText.getText("TXT_KEY_AIR_ASSAULT_BRIGADE_POPUP_TITLE", ()) + u"</font>"
		szBody = u"<font=3>" + localText.getText("TXT_KEY_SELECT_AIR_ASSAULT_BRIGADE_ERA", ()) + u"</font>"
			
	if iTI == iMTI:
		szTitle = u"<font=4b>" + localText.getText("TXT_KEY_ARMY_POPUP_TITLE", ()) + u"</font>"
		szBody = u"<font=3>" + localText.getText("TXT_KEY_ARMY_CHECK_FAILURE", ()) + u"</font>"
		
	if iTI == iNTI:
		szTitle = u"<font=4b>" + localText.getText("TXT_KEY_FLEET_POPUP_TITLE", ()) + u"</font>"
		szBody = u"<font=3>" + localText.getText("TXT_KEY_FLEET_CHECK_FAILURE", ()) + u"</font>"	
	
	if iTI == iTATI:
		szTitle = u"<font=4b>" + localText.getText("TXT_KEY_TACTICAL_AIR_FORCE_POPUP_TITLE", ()) + u"</font>"
		szBody = u"<font=3>" + localText.getText("TXT_KEY_TACTICAL_AIR_FORCE_CHECK_FAILURE", ()) + u"</font>"	
	
	if iTI == iSATI:
		szTitle = u"<font=4b>" + localText.getText("TXT_KEY_STRATEGIC_AIR_FORCE_POPUP_TITLE", ()) + u"</font>"
		szBody = u"<font=3>" + localText.getText("TXT_KEY_STRATEGIC_AIR_FORCE_CHECK_FAILURE", ()) + u"</font>"
			
	popupInfo.setText(szTitle + u"\n\n" + szBody)
	popupInfo.addPythonButton("OK", "")
	popupInfo.addPopup(iPlayer)

	
def getNewCiv4CombinedUnit(iPlayer, xUnitType, pPlot):
	# Orion's Combined Forces Mod
	# Returns the Target Unit
	pPlayer = gc.getPlayer(iPlayer)
	iAirUnitType = gc.getInfoTypeForString("UNITCOMBAT_AIR")
	iAFID = gc.getInfoTypeForString("PROMOTION_AIR_FORCE_ID")
	
	for iLoop in range (pPlot.getNumUnits()):
		TargetUnit = pPlot.getUnit(iLoop)
		TargetUnitUnitType = TargetUnit.getUnitType()
		TargetUnitCombatType = TargetUnit.getUnitCombatType()
		
		if TargetUnitUnitType == xUnitType:			
			# Isolate the new Air Force unit from other Air units
			if TargetUnitCombatType == iAirUnitType:
				# Does the unit have the AF ID promotion?
				if TargetUnit.isHasPromotion(iAFID):
					break				
			
			elif TargetUnitCombatType != iAirUnitType:
				# Isolate the new Combined Unit from other Combined units
				if TargetUnit.baseCombatStr() == 1:
					break				
			
	return TargetUnit


def createCiv4CombinedUnit(iPlayer, iCombinedUnitType, AIpCity, iTI):
	# Orion's Combined Forces Mod
	# 1. Creates Army or Navy units
	# 2. Add Leadership Promotion, if the AIpCity has West Point or Annapolis National Wonder    
	pPlayer = gc.getPlayer(iPlayer)
	iHTI = gc.getInfoTypeForString("UNIT_HTI")
	iMTI = gc.getInfoTypeForString("UNIT_MTI")
	iNTI = gc.getInfoTypeForString("UNIT_NTI")	
	DefaultUnitAI = UnitAITypes.NO_UNITAI
	iX = AIpCity.getX()
	iY = AIpCity.getY()	
	
	pPlayer.initUnit(iCombinedUnitType, iX, iY, DefaultUnitAI, DirectionTypes.NO_DIRECTION)
	#CyInterface().addImmediateMessage("Initiate Army", "")
		
	if hasTIUnit(AIpCity, iPlayer, iTI):
		AICityPlot = CyMap().plot(iX, iY)
		# Get the new Combined unit
		MyNewCombinedUnit = getNewCiv4CombinedUnit(iPlayer, iCombinedUnitType, AICityPlot)
		
		if iTI == iHTI or iTI == iMTI:
			iWestPoint = gc.getInfoTypeForString("BUILDING_WEST_POINT")
			# Does the city have the West Point National Wonder?
			if AIpCity.isHasBuilding(iWestPoint):
				iLeadership = gc.getInfoTypeForString("PROMOTION_LEADERSHIP")
				# Add Leadership promotion to Combined Unit
				MyNewCombinedUnit.setHasPromotion(iLeadership, True)
		
		elif iTI == iNTI:
			iAnnapolis = gc.getInfoTypeForString("BUILDING_ANNAPOLIS")
			# Does the city have the Annapolis National Wonder?
			if AIpCity.isHasBuilding(iAnnapolis):				
				# Add Leadership promotion to Combined Unit
				MyNewCombinedUnit.setHasPromotion(iLeadership, True)
	

def	getTempAirForceUnit(iPlayer, pCity):
	# Orion's Combined Forces Mod
	# Returns the Air unit that has the Temp Air Force promotion
	pPlayer = gc.getPlayer(iPlayer)
	iX = pCity.getX()
	iY = pCity.getY()
	pCityPlot = CyMap().plot(iX, iY)
	iAFID = gc.getInfoTypeForString("PROMOTION_AIR_FORCE_ID")
	
	for iUnitLoop in range (pCityPlot.getNumUnits()):
		pAFUnit = pCityPlot.getUnit(iUnitLoop)
		
		if pAFUnit.isHasPromotion(iAFID):
			return pAFUnit
	
	
def hasTempAirForceUnit(iPlayer, iUnitType, pCity):	
	# Orion's Combined Forces Mod
	# Returns True if the player has a unit with the Temp Air Force promotion  
	# Returns False if the player does not have a unit with the Temp Air Force promotion  
	pPlayer = gc.getPlayer(iPlayer)
	iX = pCity.getX()
	iY = pCity.getY()
	pCityPlot = CyMap().plot(iX, iY)
	FoundTempAFPromotion = False
	iAFID = gc.getInfoTypeForString("PROMOTION_AIR_FORCE_ID")
	
	for iUnitLoop in range (pCityPlot.getNumUnits()):
		pAFUnit = pCityPlot.getUnit(iUnitLoop)
		pAFUnitType = pAFUnit.getUnitType()
		
		if pAFUnitType == iUnitType:
			if pAFUnit.isHasPromotion(iAFID):
				FoundTempAFPromotion = True
				break
	
	if FoundTempAFPromotion:
		return true
	
	return False

def doGetAIEraCombinedUnits(iPlayer, iSelectedEra, iTI, pUnitsToDeleted):
	# Orion's Combined Forces Mod
	# Deletes specified number of units to create a combined unit. 
	pPlayer = gc.getPlayer(iPlayer)
	iHTI = gc.getInfoTypeForString("UNIT_HTI")
	iMTI = gc.getInfoTypeForString("UNIT_MTI")
	iNTI = gc.getInfoTypeForString("UNIT_NTI")
	iTATI = gc.getInfoTypeForString("UNIT_TATI")
	iSATI = gc.getInfoTypeForString("UNIT_SATI")
	pCombinedUnitMoves = 10
	ProcessUnit = False
	AirUnit = False
			
	# Army or Air Force
	global iTotalPromotions
	global iTotalUnitsDivider
	global pCombinedUnitCombat
	global iCombinedUnitType
	
	if iTI == iHTI:	
		MydCombatList = dAirAssaultBrigadeList
		MyCombinedTxt = "TXT_KEY_MESSAGE_AIR_ASSAULT_BRIGADE_BUILT"
	
	if iTI == iMTI:		
		MydCombatList = dArmyCombatList
		MyCombinedTxt = "TXT_KEY_MESSAGE_ARMY_BUILT"
		
	if iTI == iNTI:		
		MydCombatList = dNavyCombatList
		MyCombinedTxt = "TXT_KEY_MESSAGE_FLEET_BUILT"
	
	if iTI == iTATI:
		MydCombatList = dAirForceCombatList
		MyCombinedTxt = "TXT_KEY_MESSAGE_TACTICAL_AIR_FORCE_BUILT"

	if iTI == iSATI:
		#CyInterface().addImmediateMessage("SATI", "")
		MydCombatList = dAirForceCombatList
		MyCombinedTxt = "TXT_KEY_MESSAGE_STRATEGIC_AIR_FORCE_BUILT"
	
	# Get City with specified Training Instructor
	MyZCity = getTICity(iPlayer, iTI)
	iX = MyZCity.getX()
	iY = MyZCity.getY()
	pPlot = CyMap().plot(iX, iY)	
	
	# Check each Unit on plot
	for iUnitLoop in range (pPlot.getNumUnits()):
		# Identify the Unit 
		pUnitZ = pPlot.getUnit(iUnitLoop)
		pUnitZType = pUnitZ.getUnitType()		
		pUnitZCombatType = pUnitZ.getUnitCombatType()		
							
		# Make sure only undamaged units can be selected
		if not pUnitZ.isHurt():
			# Exclude units not in Combat List
			if isTypeInList(pUnitZCombatType, MydCombatList):
				# Exclude units with prohibited AITypes
				if not isTypeInList(pUnitZType, dProhibitedAITypeList):				
					# Get the Era for the Unit
					iEra = getUnitEra(pUnitZType)
					
					#CyInterface().addImmediateMessage("A", "")
					# If building an Air Force
					if MydCombatList == dAirForceCombatList:					
						# Air Units
						AirUnit = True
						iAFID = gc.getInfoTypeForString("PROMOTION_AIR_FORCE_ID")
						
						# Is the Unit Era equal to the Selected Era
						if iEra == iSelectedEra:
							####### Got the right unit #######
							# If Tactical Air Training Instructor.
							if iTI == iTATI:
								#CyInterface().addImmediateMessage("Tactical Air", "")								
								# Look for Fighters
								pUnitZIntercept = gc.getUnitInfo(pUnitZType).getInterceptionProbability()
								
								if not pUnitZIntercept == 0:
									####### Got the right unit #######
									ProcessUnit = True
									
									# If there is no unit with temp Air Force Promotion
									if not hasTempAirForceUnit(iPlayer, pUnitZType, MyZCity):
										iAFAcademy = gc.getInfoTypeForString("BUILDING_AF_ACADEMY")
										DefaultUnitAI = UnitAITypes.NO_UNITAI
										# Create New Tactical Air Force Unit
										CreateNewAirForceUnit = pPlayer.initUnit(pUnitZType, iX, iY, DefaultUnitAI, DirectionTypes.NO_DIRECTION)
										# Identify New Tactical Air Force unit
										CreateNewAirForceUnit.setHasPromotion(iAFID, True)
										# Does the city have the Air Force Accademy National Wonder?
										if MyZCity.isHasBuilding(iAFAcademy):
											iLeadership = gc.getInfoTypeForString("PROMOTION_LEADERSHIP")
											
											# Add Leadership promotion
											CreateNewAirForceUnit.setHasPromotion(iLeadership, True)
																		
									break
							
							# If Strategic Air Training Instructor.
							if iTI == iSATI:
								#CyInterface().addImmediateMessage("Strategic Air", "")
								# Look for Bombers
								pUnitZIntercept = gc.getUnitInfo(pUnitZType).getInterceptionProbability()
								
								if pUnitZIntercept == 0:
									####### Got the right unit #######
									ProcessUnit = True
									
									# If there is no unit with temp Air Force Promotion
									if not hasTempAirForceUnit(iPlayer, pUnitZType, MyZCity):
										iAFAcademy = gc.getInfoTypeForString("BUILDING_AF_ACADEMY")
										DefaultUnitAI = UnitAITypes.NO_UNITAI
										# Create New Strategic Air Force Unit
										CreateNewAirForceUnit = pPlayer.initUnit(pUnitZType, iX, iY, DefaultUnitAI, DirectionTypes.NO_DIRECTION)
										# Identify New Strategic Air Force unit
										CreateNewAirForceUnit.setHasPromotion(iAFID, True)
										# Does the city have the Air Force Accademy National Wonder?
										if MyZCity.isHasBuilding(iAFAcademy):
											iLeadership = gc.getInfoTypeForString("PROMOTION_LEADERSHIP")
											
											# Add Leadership promotion
											CreateNewAirForceUnit.setHasPromotion(iLeadership, True)
																			
									break
					
					if MydCombatList != dAirForceCombatList:
						if iEra <= iSelectedEra:
							####### Got the right unit #######
							ProcessUnit = True
							# Identify the virgin Combined Unit
							MyNewCombinedUnit = getNewCiv4CombinedUnit(iPlayer, iCombinedUnitType, pPlot)
							# 1. Record Combat Strength
							pCombinedUnitCombat += pUnitZ.baseCombatStr()
							#CyInterface().addImmediateMessage(str(pCombinedUnitCombat), "")
							# 2. Get unit moves
							pUnitZMovement = gc.getUnitInfo(pUnitZType).getMoves()
							
							# Is the unit movement less than pCombinedUnitMoves
							if pUnitZMovement < pCombinedUnitMoves:
								# 5. Update pCombinedUnitMoves with smaller value
								pCombinedUnitMoves = pUnitZMovement
							
							break
								
	#	CyInterface().addImmediateMessage(str(pCombinedUnitCombat), "")	
	if ProcessUnit:
		# 1. Record the total number of Promotions for the selected unit
		iTotalPromotions += OVPromotionUtils.getTotalUnitPromotions(pUnitZ)
		
		if not AirUnit:
			# 2. Transfer all promotions from unit to CombinedForces Unit
			doTransferPromotions(pUnitZ, MyNewCombinedUnit)
		
		if AirUnit:
			# Get existing New Air Force Unit
			MyNewAirForceUnit = getTempAirForceUnit(iPlayer, MyZCity)		
			# 2. Transfer all promotions from unit to CombinedForces Unit
			doTransferPromotions(pUnitZ, MyNewAirForceUnit)
		
		# Delete Unit
		#CyInterface().addImmediateMessage("Killed!", "")
		pUnitZ.kill( 0, -1 )
			
		if pUnitsToDeleted == iTotalUnitsDivider:
			# Calculate the promotion rate
			iSinglePromotionRate = OVPromotionUtils.getPromotionRate(iTotalUnitsDivider, iTotalPromotions)		
			# Delete the Training Instructor		
			doDeleteTI(iTI, iPlayer)
			#CyInterface().addImmediateMessage("A", "")
			
			if not AirUnit:
				# Look for Army or Navy unit
				if MyNewCombinedUnit.baseCombatStr() == 1:
					# 1. Run Promotion repeals for the combined unit.
					doPromotionRepeals(MyZCity, MyNewCombinedUnit, iSinglePromotionRate)
					# 2. Adjust accumulated strength to the correct value
					RealStrength = pCombinedUnitCombat - 1
					# 3. Set the accumulated strength for the combined unit.
					MyNewCombinedUnit.setBaseCombatStr(RealStrength)
					
					# 4. Set Movement points for new Combined Unit
					if pCombinedUnitMoves > 1:
						MyInt = int(pCombinedUnitMoves) - 1
						MovementPromotion = "PROMOTION_MOVEMENT" + str(MyInt)
						iMovement = gc.getInfoTypeForString(str(MovementPromotion))
						MyNewCombinedUnit.setHasPromotion(iMovement, True)
									
					if pPlayer.isHuman():
						CyInterface().addMessage(CyGame().getActivePlayer(),False,25,CyTranslator().getText(MyCombinedTxt,(pCity.getName(),)),"AS2D_WELOVEKING",InterfaceMessageTypes.MESSAGE_TYPE_INFO, MyNewCombinedUnit.getButton(),ColorTypes(8), iX, iY, True, True)
					
			elif AirUnit:
				MyNewAirForceUnit = getTempAirForceUnit(iPlayer, MyZCity)
				# Make sure this is not a None Type object
				if MyNewAirForceUnit != None:
					# Identify the Unit
					if MyNewAirForceUnit.isHasPromotion(iAFID):
						#CyInterface().addImmediateMessage("Has Promotion", "")
						# 1. Run Promotion repeals for new Air Force Unit
						doPromotionRepeals(MyZCity, MyNewAirForceUnit, iSinglePromotionRate)
						iAirCombatPromotion1 = gc.getInfoTypeForString("PROMOTION_AIR_FORCE_COMBAT1")
						iAirCombatPromotion2 = gc.getInfoTypeForString("PROMOTION_AIR_FORCE_COMBAT2")
						iAirCombatPromotion3 = gc.getInfoTypeForString("PROMOTION_AIR_FORCE_COMBAT3")
						# 2. Set the correct Air Combat strength promotion based upon the iTotalUnitsDivider value.
						if iTotalUnitsDivider == 3:
							MyNewAirForceUnit.setHasPromotion(iAirCombatPromotion1, True)
						
						if iTotalUnitsDivider == 4:
							MyNewAirForceUnit.setHasPromotion(iAirCombatPromotion2, True)
						
						if iTotalUnitsDivider == 5:
							MyNewAirForceUnit.setHasPromotion(iAirCombatPromotion3, True)
						
						# 3. Run Temp AF Promotion repeal (Note: Must be last edit)
						MyNewAirForceUnit.setHasPromotion(iAFID, false)
						if pPlayer.isHuman():
							CyInterface().addMessage(CyGame().getActivePlayer(),False,25,CyTranslator().getText(MyCombinedTxt,(pCity.getName(),)),"AS2D_WELOVEKING",InterfaceMessageTypes.MESSAGE_TYPE_INFO, MyNewAirForceUnit.getButton(),ColorTypes(8), iX, iY, True, True)
							# Done

def doPickUnitsToCombinePopup(iPlayer, iSelectedEra, iTI, iTotalUnitsDivider):
	# Orion's Combined Forces Mod
	# Displays popup to select units for new combined unit.
	#CyInterface().addImmediateMessage(str(iMaxUnits), "")
	pPlayer = gc.getPlayer(iPlayer)
	iRefCityList = PyPlayer(iPlayer).getCityList()
	iHTI = gc.getInfoTypeForString("UNIT_HTI")
	iMTI = gc.getInfoTypeForString("UNIT_MTI")
	iNTI = gc.getInfoTypeForString("UNIT_NTI")
	iTATI = gc.getInfoTypeForString("UNIT_TATI")
	iSATI = gc.getInfoTypeForString("UNIT_SATI")
	popupInfo = CyPopupInfo()
	popupInfo.setButtonPopupType(ButtonPopupTypes.BUTTONPOPUP_PYTHON)
	
	if iTI == iHTI:
		szTitle = u"<font=4b>" + localText.getText("TXT_KEY_AIR_ASSAULT_BRIGADE_POPUP_TITLE", ()) + u"</font>"
		szBody = u"<font=3>" + localText.getText("TXT_KEY_SELECT_AIR_ASSAULT_BRIGADE_ERA", ()) + u"</font>"
		popupInfo.setData3(iHTI)
		MydCombatList = dAirAssaultBrigadeList
		
	if iTI == iMTI:
		szTitle = u"<font=4b>" + localText.getText("TXT_KEY_ARMY_POPUP_TITLE", ()) + u"</font>"
		szBody = u"<font=3>" + localText.getText("TXT_KEY_SELECT_ARMY_UNITS", ()) + u"</font>"
		popupInfo.setData3(iMTI)
		MydCombatList = dArmyCombatList
		
	if iTI == iNTI:
		szTitle = u"<font=4b>" + localText.getText("TXT_KEY_FLEET_POPUP_TITLE", ()) + u"</font>"
		szBody = u"<font=3>" + localText.getText("TXT_KEY_SELECT_FLEET_UNITS", ()) + u"</font>"	
		popupInfo.setData3(iNTI)
		MydCombatList = dNavyCombatList
		
	if iTI == iTATI:
		szTitle = u"<font=4b>" + localText.getText("TXT_KEY_TACTICAL_AIR_FORCE_POPUP_TITLE", ()) + u"</font>"
		szBody = u"<font=3>" + localText.getText("TXT_KEY_SELECT_TACTICAL_AIR_FORCE_UNITS", ()) + u"</font>"	
		popupInfo.setData3(iTATI)
		MydCombatList = dAirForceCombatList
		
	if iTI == iSATI:
		szTitle = u"<font=4b>" + localText.getText("TXT_KEY_STRATEGIC_AIR_FORCE_POPUP_TITLE", ()) + u"</font>"
		szBody = u"<font=3>" + localText.getText("TXT_KEY_SELECT_STRATEGIC_AIR_FORCE_UNITS", ()) + u"</font>"	
		popupInfo.setData3(iSATI)
		MydCombatList = dAirForceCombatList		
	
	popupInfo.setOnClickedPythonCallback("selectUnitCallback")
	popupInfo.setText(szTitle + u"\n\n" + szBody)
	popupInfo.setData1(iPlayer)
		
	#Populate the Popup
	for pyCity in iRefCityList:
		pCity = pyCity.GetCy()
		
		if pCity != None:
			iX = pCity.getX()
			iY = pCity.getY()
			pPlot = CyMap().plot(iX, iY)
			
			# Does the city have an MTI?			
			if hasTIUnit(pCity, iPlayer, iTI):	
				# Check each Unit on plot
				for iUnitLoop in range (pPlot.getNumUnits()):
					# Identify the Unit
					pUnitZ = pPlot.getUnit(iUnitLoop)						
					pUnitZType = pUnitZ.getUnitType()
					pUnitZCombatType = pUnitZ.getUnitCombatType()
					pUnitZAIType = pUnitZ.getUnitAIType()
					#pUnitZName = gc.getUnitInfo(pUnitZType).getType()
					
					# Make sure only undamaged units can be selected
					if not pUnitZ.isHurt():
						# Exclude units not in Combat List			
						if isTypeInList(pUnitZCombatType, MydCombatList):
							# Exclude units with prohibited AITypes
							if not isTypeInList(pUnitZAIType, dProhibitedAITypeList):					
								#CyInterface().addImmediateMessage("X", "")
								# Get the Era for the Unit
								iEra = getUnitEra(pUnitZType)
								
								if MydCombatList == dAirForceCombatList:
									# Is the Unit Era equal to the Selected Era
									if iEra == iSelectedEra:								
										# Does the plot have the min number of units for this pUnitZType
										if getPlotAirUnitCount(pPlot, pUnitZType, iTotalUnitsDivider, iTI):
											# Store the era for later use
											popupInfo.setData2(iSelectedEra)
											# Display filtered list of units in popup
											popupInfo.addPythonButton(gc.getUnitInfo(pUnitZType).getDescription(), gc.getUnitInfo(pUnitZType).getButton())
											#CyInterface().addImmediateMessage("Here", "")
							
							if MydCombatList != dAirForceCombatList:
								# Is the Unit Era equal to or less than the Selected Era
								if iEra <= iSelectedEra:								
									# Store the era for later use
									popupInfo.setData2(iSelectedEra)
									# Display filtered list of units in popup
									popupInfo.addPythonButton(gc.getUnitInfo(pUnitZType).getDescription(), gc.getUnitInfo(pUnitZType).getButton())
									#CyInterface().addImmediateMessage("Here", "")
										
	popupInfo.addPopup(iPlayer)


def addPopup(szTitle, szText):	
	# Orion's Combined Forces Mod
	# Don't display popups for autoplay games
	if (gc.getPlayer(CyGame().getActivePlayer()).isAlive()):
	
		popup = PyPopup.PyPopup(-1)
		popup.setHeaderString(szTitle)
		popup.setBodyString(szText)
		popup.launch(true, PopupStates.POPUPSTATE_QUEUED)

	
def doCombinedUnitPreChecks(iPlayer, iSelectedEra, pCity, iTI):
	# Orion's Combined Forces Mod
	# Perform checks and obtains values for combined unit
	# ********** For AI Players ************
	pPlayer = gc.getPlayer(iPlayer)
	iHTI = gc.getInfoTypeForString("UNIT_HTI")
	iMTI = gc.getInfoTypeForString("UNIT_MTI")
	iNTI = gc.getInfoTypeForString("UNIT_NTI")
	iTATI = gc.getInfoTypeForString("UNIT_TATI")
	iSATI = gc.getInfoTypeForString("UNIT_SATI")
	iCurrentEra = pPlayer.getCurrentEra()
	
	# Run Checks	
	#CyInterface().addImmediateMessage("A", "")
				
	global iCombinedUnitType
	global pCombinedUnitCombat
	global pUnitsToDelete
	global iTotalPromotions
	global iTotalUnitsDivider	
	
	MyMaxUnits = getMaxUnits(iCurrentEra, iSelectedEra)
	
	if MyMaxUnits != None:		
		
		iTotalPromotions = 0
		pCombinedUnitCombat = 1
		pUnitsToDelete = MyMaxUnits + 1
		iTotalUnitsDivider = MyMaxUnits
		iMaxDiv = iTotalUnitsDivider + 1
		
		# Helicopter Training Instructor
		if iTI == iHTI:
			# Get ArmyTpye based on selected Era
			MyCombinedUnitType = getCombinedUnitType(iSelectedEra, iTI)
			iCombinedUnitType = MyCombinedUnitType
			# Create Combined Unit
			createCiv4CombinedUnit(iPlayer, iCombinedUnitType, pCity, iTI)
		
		# Army Training Instructor
		if iTI == iMTI:
			# Get ArmyTpye based on selected Era
			MyCombinedUnitType = getCombinedUnitType(iSelectedEra, iMTI)
			iCombinedUnitType = MyCombinedUnitType
			# Create Combined Unit
			createCiv4CombinedUnit(iPlayer, iCombinedUnitType, pCity, iMTI)
			
		# Navy Training Instructor
		if iTI == iNTI:
			# Get FleetTpye based on selected Era
			MyCombinedUnitType = getCombinedUnitType(iSelectedEra, iNTI)
			iCombinedUnitType = MyCombinedUnitType
			# Create Combined Unit
			createCiv4CombinedUnit(iPlayer, iCombinedUnitType, pCity, iNTI)
							
		# Air Force Training Instructor
		if iTI == iTATI or iTI == iSATI:
			iCombinedUnitType = -1
						
		for iUnitsLeft in range(1, pUnitsToDelete):
			#CyInterface().addImmediateMessage("A", "")
			doGetAIEraCombinedUnits(iPlayer, iSelectedEra, iTI, iUnitsLeft)
	
				
def getMaxUnits(iCurrentEra, iSelectedEra):
	# Orion's Combined Forces Mod
	# Returns the Maximum number of units that can be added into the Combined Unit	
	for szEra, (iAncient, iClassical, iMedieval, iRenaissance, iIndustrial, iModern, iFuture) in dMaxUnitsData.iteritems():
		iTechEra = gc.getInfoTypeForString(szEra)
		# Select the data line that matches the player's current ERA
		if iTechEra == iCurrentEra:
			break
		
	# 0 = ERA_ANCIENT
	if iSelectedEra == 0:
		return iAncient
		
	# 1 = ERA_CLASSICAL
	if iSelectedEra == 1:
		return iClassical
		
	# 2 = ERA_MEDIEVAL
	if iSelectedEra == 2:
		return iMedieval
		
	# 3 = ERA_RENAISSANCE
	if iSelectedEra == 3:
		return iRenaissance
		
	# 4 = ERA_INDUSTRIAL
	if iSelectedEra == 4:
		return iIndustrial
		
	# 5 = ERA_MODERN
	if iSelectedEra == 5:
		return iModern
		
	# 6 = ERA_FUTURE
	if iSelectedEra == 6:
		return iFuture
		

def getCombinedUnitType(iEra, iTI):	
	# Orion's Combined Forces Mod
	# Returns the unit type for the combined uit.
	iHTI = gc.getInfoTypeForString("UNIT_HTI")
	iMTI = gc.getInfoTypeForString("UNIT_MTI")
	iNTI = gc.getInfoTypeForString("UNIT_NTI")	
	iTATI = gc.getInfoTypeForString("UNIT_TATI")
	iSATI = gc.getInfoTypeForString("UNIT_SATI")
	#Set a Temporary value
	xCombinedUnitType = gc.getInfoTypeForString("UNIT_ANCIENT_ARMY")	
	
	# 0 = ERA_ANCIENT
	if iEra == 0:
		if iTI == iMTI:
			xCombinedUnitType = gc.getInfoTypeForString("UNIT_ANCIENT_ARMY")
			
		if iTI == iNTI:	
			xCombinedUnitType = gc.getInfoTypeForString("UNIT_ANCIENT_FLEET")
			
		if iTI == iTATI or iTI == iSATI:
			xCombinedUnitType = -1
		
	# 1 = ERA_CLASSICAL
	if iEra == 1:
		if iTI == iMTI:
			xCombinedUnitType = gc.getInfoTypeForString("UNIT_CLASSICAL_ARMY")
			
		if iTI == iNTI:
			xCombinedUnitType = gc.getInfoTypeForString("UNIT_CLASSICAL_FLEET")
			
		if iTI == iTATI or iTI == iSATI:
			xCombinedUnitType = -1	
		
	# 2 = ERA_MEDIEVAL
	if iEra == 2:
		if iTI == iMTI:
			xCombinedUnitType = gc.getInfoTypeForString("UNIT_MEDIEVAL_ARMY")
			
		if iTI == iNTI:
			xCombinedUnitType = gc.getInfoTypeForString("UNIT_MEDIEVAL_FLEET")
			
		if iTI == iTATI or iTI == iSATI:
			xCombinedUnitType = -1	
			
	# 3 = ERA_RENAISSANCE
	if iEra == 3:
		if iTI == iMTI:
			xCombinedUnitType = gc.getInfoTypeForString("UNIT_RENAISSANCE_ARMY")
			
		if iTI == iNTI:
			xCombinedUnitType = gc.getInfoTypeForString("UNIT_RENAISSANCE_FLEET")
			
		if iTI == iTATI or iTI == iSATI:
			xCombinedUnitType = -1	
		
	# 4 = ERA_INDUSTRIAL
	if iEra == 4:
		if iTI == iMTI:
			xCombinedUnitType = gc.getInfoTypeForString("UNIT_INDUSTRIAL_ARMY")
			
		if iTI == iNTI:	
			xCombinedUnitType = gc.getInfoTypeForString("UNIT_INDUSTRIAL_FLEET")

		if iTI == iTATI or iTI == iSATI:
			xCombinedUnitType = -1		
		
	# 5 = ERA_MODERN
	if iEra == 5:
		if iTI == iHTI:
			xCombinedUnitType = gc.getInfoTypeForString("UNIT_MODERN_AABRIGADE")
			
		if iTI == iMTI:
			xCombinedUnitType = gc.getInfoTypeForString("UNIT_MODERN_ARMY")	

		if iTI == iNTI:	
			xCombinedUnitType = gc.getInfoTypeForString("UNIT_MODERN_FLEET")
			
		if iTI == iTATI or iTI == iSATI:
			xCombinedUnitType = -1
			
	# 6 = ERA_FUTURE
	if iEra == 6:
		if iTI == iHTI:
			xCombinedUnitType = gc.getInfoTypeForString("UNIT_FUTURE_AABRIGADE")
			
		if iTI == iMTI:
			xCombinedUnitType = gc.getInfoTypeForString("UNIT_FUTURE_ARMY")
			
		if iTI == iNTI:
			xCombinedUnitType = gc.getInfoTypeForString("UNIT_FUTURE_FLEET")
			
		if iTI == iTATI or iTI == iSATI:
			xCombinedUnitType = -1
		
	
	return xCombinedUnitType

	
def doTransferPromotions(FromUnit, ToUnit):
	# Orion's Combined Forces Mod
	# Transfers promotions from selected unit to the combined unit
	iGreatGeneral = gc.getInfoTypeForString("UNIT_GREAT_GENERAL")
	
	# Loop through promotions
	for iProm in range(gc.getNumPromotionInfos()):
		# Does the unit have this promotion
		if FromUnit.isHasPromotion(iProm):
			# Combined unit must not already have this promotion
			if not ToUnit.isHasPromotion(iProm):
				# Transfer promotion to the New Combined Unit
				ToUnit.setHasPromotion(iProm, True)
				# Get the Leader Type
				FromUnitLeaderType = FromUnit.getLeaderUnitType()
				# Does the unit have a leader?
				if FromUnitLeaderType != -1:
					# Get the Leader Name
					FromUnitName = FromUnit.getNameNoDesc()
					#CyInterface().addImmediateMessage(str(FromUnitName), "")
					# Get the Leader Type for Combined Unit
					MyCombinedUnitLeaderType = ToUnit.getLeaderUnitType()
					# Make sure the Army unit does not already have a leader
					if MyCombinedUnitLeaderType == -1:						
						ToUnit.setLeaderUnitType(iGreatGeneral)
						ToUnit.setName(FromUnitName)

						
def doPromotionRepeals(pCity, MyNewCombinedUnit, iSinglePromotionRate):
	# Orion's Combined Forces Mod 
	# Removes promotions from new Combined Forces unit
	# Promotion removal is based upon randum chance 
	iSentry = gc.getInfoTypeForString("PROMOTION_SENTRY")
	iWestPoint = gc.getInfoTypeForString("BUILDING_WEST_POINT")
	iAnnapolis = gc.getInfoTypeForString("BUILDING_ANNAPOLIS")
	iAFAcademy = gc.getInfoTypeForString("BUILDING_AF_ACADEMY")
	iLeadership = gc.getInfoTypeForString("PROMOTION_LEADERSHIP")
	iAFID = gc.getInfoTypeForString("PROMOTION_AIR_FORCE_ID")	
	OVPromotionUtils.runGroupListCheck(MyNewCombinedUnit)
	
	for iProm in range(gc.getNumPromotionInfos()):
		if MyNewCombinedUnit.isHasPromotion(iProm):
			PromotionGroup = OVPromotionUtils.getPromotionGroup(iProm)
						
			# Select promotions that are not in a promotion group 
			if str(PromotionGroup) == "NONE":
				# Exclude these Promotions
				if iProm != iSentry and iProm != iAFID:
					#CyInterface().addImmediateMessage("X", "")
					# Run calculated chance of loosing the promotion
					if not OVPromotionUtils.doRandumChance(iSinglePromotionRate):
						# Does the city have the West Point or Annapolis National Wonder?
						if pCity.isHasBuilding(iWestPoint) or pCity.isHasBuilding(iAnnapolis) or pCity.isHasBuilding(iAFAcademy):
							# Exclude the leadership promotion
							if iProm != iLeadership:
								# Remove Promotion
								MyNewCombinedUnit.setHasPromotion(iProm, false)
								
						else:
							# Remove Promotion
							MyNewCombinedUnit.setHasPromotion(iProm, false)


def hasNavyRallyPointCity(ePlayer):
	# Orion's Combined Forces Mod
	# Returns True if the player has a Navy Rally Point city
	# Returns False if the player does not have a Navy Rally Point city
	lCoastalCities = getCoastalCities(ePlayer)
	iHarbor = gc.getInfoTypeForString("BUILDING_HARBOR")
	pCityConnected = False

	for pCoastalCity in lCoastalCities:
		pCostalCityPopulation = pCoastalCity.getPopulation()
		
		if pCostalCityPopulation > 5:
			if pCoastalCity.isHasBuilding(iHarbor):
				if pCoastalCity.isCoastal(10):
					#pOcean = pCoastalCity.waterArea()
					for pTargetCity in lCoastalCities:
						if pTargetCity != pCoastalCity:
							if pTargetCity.isConnectedTo(pCoastalCity):
								#tOcean = pTargetCity.waterArea()
								#RallyPointCityName = pCoastalCity.getName()								
								#CyInterface().addImmediateMessage("Connected", "")
								pCityConnected = True
	
	if pCityConnected:
		return True
	
	return False


def getNavyRallyPointCity(ePlayer):
	# Orion's Combined Forces Mod
	# Returns the coastal city with the most number of connections	
	lCoastalCities = getCoastalCities(ePlayer)
	iHighestNum = 0
	iHarbor = gc.getInfoTypeForString("BUILDING_HARBOR")
	pMostConnected = None  
	MyPopulation = 0	
	
	for pCoastalCity in lCoastalCities:
		iNumConnections = 0
		pCostalCityPopulation = pCoastalCity.getPopulation()
		# Sort Coastal Cities by population
		for iCityPop in range(25, 1, -1):
			# Get Highest populated city first and if no match, proceed.
			if pCostalCityPopulation == iCityPop:
				if pCoastalCity.isHasBuilding(iHarbor):
					if pCoastalCity.isCoastal(10):
						# Loop through Coastal Cities
						for pTargetCity in lCoastalCities:
							pTargetCityPop = pTargetCity.getPopulation()
							if pTargetCity != pCoastalCity:
								if pCostalCityPopulation >= pTargetCityPop:
									if pCoastalCity.isConnectedTo(pTargetCity):
										#CyInterface().addImmediateMessage("A", "")
										#if pOcean == pTargetCity.waterArea():
										iNumConnections += 1
						
						if iNumConnections > iHighestNum:
							iHighestNum = iNumConnections
							pMostConnected = pCoastalCity
	
	return pMostConnected
	
	
def getCoastalCities(ePlayer):
	# Orion's Combined Forces Mod
    # Returns a list array of costal cities next to a mass of water of at least 10 map plots.
   	lCoastalCities = list()
	iCityList = PyPlayer(ePlayer).getCityList()
	
	for pyCity in iCityList:
		pCity = pyCity.GetCy()
		
		if pCity != None:
			# Is this a coastal city?
			if pCity.isCoastal(10):
				lCoastalCities.append(pCity)
			
	return lCoastalCities
	
	
def doNavyRedeployToRallyPoint(ePlayer):
	# Orion's Combined Forces Mod
	# Deploys all old combat ships to Rally Point City
	if hasNavyRallyPointCity(ePlayer):
		if getNavyRallyPointCity(ePlayer) != None:
			AIRallyPointCity = getNavyRallyPointCity(ePlayer)
			pPlayer = gc.getPlayer(ePlayer)
			iCurrentEra = pPlayer.getCurrentEra()
			iReferUnitList = PyPlayer(ePlayer).getUnitList()
			#CyInterface().addImmediateMessage("A", "")
			
			# Loop through Eras
			for iMothBallEra in range(gc.getNumEraInfos()):
				if not iMothBallEra > iCurrentEra - 2:
					# Loop through the player's units
					for pyUnit in iReferUnitList:
						pyUnitType = pyUnit.getUnitType()
						pyUnitCombatType = pyUnit.getUnitCombatType()
						# Exclude units not in Combat List
						if isTypeInList(pyUnitCombatType, dNavyCombatList):
							# Exclude units with prohibited AITypes
							if not isTypeInList(pyUnitType, dProhibitedAITypeList):
								# Get the Era for the Unit
								iUnitEra = getUnitEra(pyUnitType)
								# Is the Unit Era equal to or less than the Mothball Era
								if iUnitEra <= iMothBallEra:
									# Find route to City
									if pyUnit.generatePath(AIRallyPointCity.plot(), 0, False, None):
										if pyUnit.getX() != AIRallyPointCity.getX() or pyUnit.getY() != AIRallyPointCity.getY():
											# Move the Unit to the city, where Fleet can be built.
											pyUnit.getGroup().pushMission(MissionTypes.MISSION_MOVE_TO, AIRallyPointCity.getX(), AIRallyPointCity.getY(), 0, False, True, MissionAITypes.NO_MISSIONAI, pyUnit.plot(), pyUnit)


def doDeleteTI(iTI, iPlayer):
	# Orion's Combined Forces Mod
	# Deletes the specified Training Instructor.
	pCity = getTICity(iPlayer, iTI)
	iXX = pCity.getX()
	iYY = pCity.getY()
	xPlot = CyMap().plot(iXX, iYY)
	
	# Check each Unit on plot
	for iLoop in range (xPlot.getNumUnits()):
		# Identify the Unit
		xUnit = xPlot.getUnit(iLoop)
		xUnitType = xUnit.getUnitType()
		# If this unit is the specified Training Instructor
		if xUnitType == iTI:
			# Unit expended -- Delete Training Instructor Unit
			#CyInterface().addImmediateMessage("Kill Unit", "")
			xUnit.kill(-1, -1)
			break


def getTICity(iPlayer, iTI):
	# Orion's Combined Forces Mod
	# Returns the city where the specified training Instructor is currently located.
	# Loop through the City List
	iRefCityList = PyPlayer(iPlayer).getCityList()
	
	for pyCity in iRefCityList:
		pCity = pyCity.GetCy()
		
		if pCity != None:		
			#Get Plot			
			# Does the city have the specified TI?			
			if hasTIUnit(pCity, iPlayer, iTI):
				#CyInterface().addImmediateMessage("THIS CITY", "")
				return pCity
	
# End Orion's Combined Forces Mod Functions

