# Sid Meier's Civilization 4
# Copyright Firaxis Games 2007
# CombinedForcesEvents
# Orion's Combined Forces Mod
# Modular python project to eliminate or reduce merging tasks

from CvPythonExtensions import *
import BugUtil
import BugCore
import CvUtil
import CvScreensInterface
import CvDebugTools
import CvWBPopups
import PyHelpers
import Popup as PyPopup
import CvCameraControls
import CvTopCivs
import sys
import CvWorldBuilderScreen
import CvAdvisorUtils
import CvTechChooser
import CombinedForces
import PyHelpers

# for C++ compatibility
false=False
true=True

# globals
gc = CyGlobalContext()
PyPlayer = PyHelpers.PyPlayer
PyInfo = PyHelpers.PyInfo
PyCity = PyHelpers.PyCity
PyGame = PyHelpers.PyGame


class CombinedForcesEvents:
	def __init__(self, eventMgr):
# ###############################################################################
#		Important!!!!															#
#		Each function added from CvEventManager.py must be initialized here 	#
# ###############################################################################
		self.__LOG_MOVEMENT = 0
		self.__LOG_UNITBUILD = 0
		self.eventMgr = eventMgr
		
		eventMgr.addEventHandler("BeginPlayerTurn", self.onBeginPlayerTurn)
		eventMgr.addEventHandler("unitBuilt", self.onUnitBuilt)	
	
	def onBeginPlayerTurn(self, argsList):
		'Called at the beginning of a players turn'
		iGameTurn, iPlayer = argsList
		pPlayer = gc.getPlayer(iPlayer)
		
		# Make sure the player is an AI player
		if not pPlayer.isNone() and not pPlayer.isBarbarian() and not pPlayer.isHuman() and pPlayer.isAlive():
			# Redeploy all navy mothball ships to rally point city 
			CombinedForces.doNavyRedeployToRallyPoint(iPlayer)
		
		
	def onUnitBuilt(self, argsList):
		'Unit Completed'
		city = argsList[0]
		unit = argsList[1]
		iOwner = city.getOwner()
		player = PyPlayer(iOwner)
	# Combined Forces Mod
		pPlayer = gc.getPlayer(iOwner)
		UnitType = unit.getUnitType()
				
		if not pPlayer.isNone() and not pPlayer.isBarbarian() and not pPlayer.isHuman( ) and pPlayer.isAlive():
			if CombinedForces.isTypeInList(UnitType, CombinedForces.dTITypeList):
				#CyInterface().addImmediateMessage("B", "")
				CombinedForces.doTIPlacementAI(unit)
		
	# End Combined Forces Mod

		CvAdvisorUtils.unitBuiltFeats(city, unit)
		
		if (not self.__LOG_UNITBUILD):
			return
		CvUtil.pyPrint('%s was finished by Player %d Civilization %s' 
			%(PyInfo.UnitInfo(unit.getUnitType()).getDescription(), player.getID(), player.getCivilizationName()))				
					
					
	