# # Sid Meier's Civilization 4
# # Copyright Firaxis Games 2007
# # CombinedForcesGameUtils

import CvUtil
import BugUtil
import BugCore
from CvPythonExtensions import *
import CvEventInterface
import CvGameUtils
import Popup as PyPopup
import PyHelpers
import GodsOfOld
import CombinedForces
import OVPromotionUtils

# for C++ compatibility
false=False
true=True

# globals
gc = CyGlobalContext()
PyPlayer = PyHelpers.PyPlayer
PyInfo = PyHelpers.PyInfo
PyCity = PyHelpers.PyCity
PyGame = PyHelpers.PyGame
iHTI = gc.getInfoTypeForString("UNIT_HTI")
iTATI = gc.getInfoTypeForString("UNIT_TATI")
iSATI = gc.getInfoTypeForString("UNIT_SATI")
iNTI = gc.getInfoTypeForString("UNIT_NTI")
iMTI = gc.getInfoTypeForString("UNIT_MTI")


dTIList1 = [
		iMTI,
		iNTI
	]
		
dTIList2 = [
		iNTI,
		iMTI
	]
		
		
dTIList3 = [
		iHTI,
		iTATI,
		iSATI,
		iNTI,
		iMTI
	]
		
dTIList4 = [
		iHTI,
		iSATI,
		iTATI,
		iMTI,
		iNTI
	]
		
dTIList5 = [
		iHTI,
		iMTI,
		iNTI,
		iTATI,
		iSATI
	]
		
dTIList6 = [
		iHTI,
		iNTI,
		iMTI,
		iSATI,
		iTATI
	]


# Combined Forces Mod By Orion Veteran
# Modular Python to eliminate Merging 
class CombinedForcesGameUtils:
		
	def AI_chooseProduction(self,argsList):
		pCity = argsList[0]
# Combined Forces Mod
		iOwner = pCity.getOwner( )
		AIpPlayer = gc.getPlayer(iOwner)
		TIProdChance = CombinedForces.getTIProductionChance(AIpPlayer)		
		iCurrentEra = AIpPlayer.getCurrentEra()
		chance = CyGame().getSorenRandNum(100,"")
		Buildit = False
				
		if iCurrentEra < 4:
			if chance <= 60:
				MyTIList = dTIList1
			
			elif chance > 60:
				MyTIList = dTIList2
		
		elif iCurrentEra > 3:
			if chance <= 25:
				MyTIList = dTIList3
			
			elif chance > 25 and chance <= 50:
				MyTIList = dTIList4
		
			elif chance > 50 and chance <= 75:
				MyTIList = dTIList5
			
			elif chance > 75 and chance <= 100:
				MyTIList = dTIList6
		
		for iTI in MyTIList:
			if pCity.canTrain(iTI, 0, 0):
				if CombinedForces.meetsTIConditions(pCity, iTI):
					if OVPromotionUtils.doRandumChance(TIProdChance):
						Buildit = True
						break
		
		if Buildit:
			pCity.pushOrder(OrderTypes.ORDER_TRAIN, iTI, -1, False, False, False, True)
			#CyInterface().addImmediateMessage("Order Successful", "")
			return True
# End Combined Forces Mod
		
		return False
		
					
	def AI_unitUpdate(self,argsList):
		pUnit = argsList[0]
# Combined Forces Mod
		iOwner = pUnit.getOwner()
		AIpPlayer = gc.getPlayer(iOwner)
		pUnitType = pUnit.getUnitType()
		
		if not AIpPlayer.isNone() and not AIpPlayer.isBarbarian() and not AIpPlayer.isHuman( ) and AIpPlayer.isAlive():
			# If pUnit is a TI
			if CombinedForces.isTypeInList(pUnitType, CombinedForces.dTITypeList):							
				CombinedForces.doTIPlacementAI(pUnit)
				#CyInterface().addImmediateMessage("Test", "")
				return True
						
# End Combined Forces Mod

		return False
	
	
	def getWidgetHelp(self, argsList):
		eWidgetType, iData1, iData2, bOption = argsList
		
		if iData1 == 691:
			return CyTranslator().getText("TXT_KEY_BUILD_AIR_ASSAULT_BRIGADE_MOUSE_OVER", ())
		
		if iData1 == 692:
			return CyTranslator().getText("TXT_KEY_BUILD_ARMY_MOUSE_OVER", ())		
		
		if iData1 == 693:
			return CyTranslator().getText("TXT_KEY_BUILD_FLEET_MOUSE_OVER", ())
			
		if iData1 == 694:
			return CyTranslator().getText("TXT_KEY_BUILD_TACTICAL_AIR_FORCE_MOUSE_OVER", ())		
		
		if iData1 == 695:
			return CyTranslator().getText("TXT_KEY_BUILD_STRATEGIC_AIR_FORCE_MOUSE_OVER", ())	
		
		return u""
		
	
	def getUpgradePriceOverride(self, argsList):
		iPlayer, iUnitID, iUnitTypeUpgrade = argsList
		pPlayer = gc.getPlayer(iPlayer)
		pUnit = pPlayer.getUnit(iUnitID)
		pUnitCombatType = pUnit.getUnitCombatType()
		iAir = gc.getInfoTypeForString("UNITCOMBAT_AIR")
		iAirCombatPromotion1 = gc.getInfoTypeForString("PROMOTION_AIR_FORCE_COMBAT1")
		iAirCombatPromotion2 = gc.getInfoTypeForString("PROMOTION_AIR_FORCE_COMBAT2")
		iAirCombatPromotion3 = gc.getInfoTypeForString("PROMOTION_AIR_FORCE_COMBAT3")
		MyMultiplier = 3
		FoundAirForce = False
		iPrice = gc.getDefineINT("BASE_UNIT_UPGRADE_COST")
		iPrice += (max(0, (pPlayer.getUnitProductionNeeded(iUnitTypeUpgrade) - pPlayer.getUnitProductionNeeded(pUnit.getUnitType()))) * gc.getDefineINT("UNIT_UPGRADE_COST_PER_PRODUCTION"))
		
		if ((not pPlayer.isHuman()) and (not pPlayer.isBarbarian())):
			pHandicapInfo = gc.getHandicapInfo(gc.getGame().getHandicapType())
			iPrice = iPrice * pHandicapInfo.getAIUnitUpgradePercent() / 100
			iPrice = iPrice * max(0, ((pHandicapInfo.getAIPerEraModifier() * pPlayer.getCurrentEra()) + 100)) / 100

		iPrice = iPrice - ((iPrice * pUnit.getUpgradeDiscount()) / 100)
		
		if pUnitCombatType == iAir:
			if pUnit.isHasPromotion(iAirCombatPromotion1):
				#CyInterface().addImmediateMessage("A", "")
				MyMultiplier = 3
				FoundAirForce = True
			
			if pUnit.isHasPromotion(iAirCombatPromotion2):
				MyMultiplier = 4
				FoundAirForce = True
									
			if pUnit.isHasPromotion(iAirCombatPromotion3):
				MyMultiplier = 5
				FoundAirForce = True
			
			if FoundAirForce:
				#return (pUnit.upgradePrice(iUnitTypeUpgrade) * MyMultiplier )
				iPrice = ((gc.getDefineINT("BASE_UNIT_UPGRADE_COST")) * MyMultiplier)
				iPrice += (((max(0, (pPlayer.getUnitProductionNeeded(iUnitTypeUpgrade) - pPlayer.getUnitProductionNeeded(pUnit.getUnitType()))) * gc.getDefineINT("UNIT_UPGRADE_COST_PER_PRODUCTION"))) * MyMultiplier)

				if ((not pPlayer.isHuman()) and (not pPlayer.isBarbarian())):
					pHandicapInfo = gc.getHandicapInfo(gc.getGame().getHandicapType())
					iPrice = ((iPrice * pHandicapInfo.getAIUnitUpgradePercent() / 100) * MyMultiplier)
					iPrice = ((iPrice * max(0, ((pHandicapInfo.getAIPerEraModifier() * pPlayer.getCurrentEra()) + 100)) / 100) * MyMultiplier)
					iPrice = ((iPrice - ((iPrice * pUnit.getUpgradeDiscount()) / 100)) * MyMultiplier)
		
		if pUnit.isHasPromotion(gc.getInfoTypeForString('PROMOTION_LEADER')):

			iPrice = 0
				
		return iPrice
		
		#else:
		#	return -1	# use default
		
# End Combined Forces Mod









